@@ -14,6 +14,7 @@ use ndarray::{
     Zip,
 };
 use ndarray::{arr0, arr1, arr2};
+use ndarray::ShapeBuilder;
 
 use test::black_box;
 
@@ -214,6 +215,17 @@ fn scalar_sum_2d_float_cutout(bench: &mut test::Bencher)
     });
 }
 
+#[bench]
+fn scalar_sum_2d_float_t_cutout(bench: &mut test::Bencher)
+{
+    let a = Array::<f32, _>::zeros((66, 66));
+    let av = a.slice(s![1..-1, 1..-1]).reversed_axes();
+    let a = black_box(av);
+    bench.iter(|| {
+        a.scalar_sum()
+    });
+}
+
 #[bench]
 fn fold_sum_i32_2d_regular(bench: &mut test::Bencher)
 {
@@ -515,19 +527,18 @@ fn add_2d_zip_strided(bench: &mut test::Bencher)
 }
 
 #[bench]
-fn add_2d_transposed(bench: &mut test::Bencher)
+fn add_2d_one_transposed(bench: &mut test::Bencher)
 {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
     let b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
-    let bv = b.view();
     bench.iter(|| {
-        a += &bv;
+        a += &b;
     });
 }
 
 #[bench]
-fn add_2d_zip_transposed(bench: &mut test::Bencher)
+fn add_2d_zip_one_transposed(bench: &mut test::Bencher)
 {
     let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
     a.swap_axes(0, 1);
@@ -537,6 +548,30 @@ fn add_2d_zip_transposed(bench: &mut test::Bencher)
     });
 }
 
+#[bench]
+fn add_2d_both_transposed(bench: &mut test::Bencher)
+{
+    let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
+    a.swap_axes(0, 1);
+    let mut b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
+    b.swap_axes(0, 1);
+    bench.iter(|| {
+        a += &b;
+    });
+}
+
+#[bench]
+fn add_2d_zip_both_transposed(bench: &mut test::Bencher)
+{
+    let mut a = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
+    a.swap_axes(0, 1);
+    let mut b = Array::<i32, _>::zeros((ADD2DSZ, ADD2DSZ));
+    b.swap_axes(0, 1);
+    bench.iter(|| {
+        Zip::from(&mut a).and(&b).apply(|a, &b| *a += b);
+    });
+}
+
 #[bench]
 fn add_2d_f32_regular(bench: &mut test::Bencher)
 {
@@ -808,6 +843,14 @@ fn equality_f32(bench: &mut test::Bencher)
     bench.iter(|| a == b);
 }
 
+#[bench]
+fn equality_f32_mixorder(bench: &mut test::Bencher)
+{
+    let a = Array::<f32, _>::zeros((64, 64));
+    let b = Array::<f32, _>::zeros((64, 64).f());
+    bench.iter(|| a == b);
+}
+
 #[bench]
 fn dot_f32_16(bench: &mut test::Bencher)
 {
